import time
import serial
import re


class AT:
    def __init__(self, port='/dev/ttyTHS1', baud_rate=115200):
        self.port = port
        self.baud_rate = baud_rate
        self.ser = None
        self.destroy_http()

    def destroy_http(self):
        try:
            self.ser = serial.Serial(self.port, self.baud_rate, timeout=1)
            self.send_at_command('AT+HTTPTERM')
        except serial.SerialException as e:
            raise Exception(f"Error initializing HTTP: {e}")

    def init_http(self):
        try:
            self.ser = serial.Serial(self.port, self.baud_rate, timeout=1)
            self.send_at_command('AT+HTTPINIT')
        except serial.SerialException:
            raise Exception(f"Error initializing")

    def open_serial_connection(self):
        try:
            self.ser = serial.Serial(self.port, self.baud_rate, timeout=1)
            return self.send_at_command('AT')[1].decode('utf-8').strip() == "OK"
        except serial.SerialException as e:
            raise Exception(f"Error opening serial connection: {e}")

    def send_at_command(self, command, wait_for_response=True):
        try:
            encoded_command = command.encode('utf-8') + b'\r\n'
            self.ser.write(encoded_command)
            print(f"Sent command: {command}")

            if wait_for_response:
                response = self.ser.readlines()
                print(f"Response: {response[1].decode('utf-8').strip()}")
                return response
            else:
                return None
        except serial.SerialException as e:
            raise Exception(f"Error sending AT command: {e}")

    # The API DOC is URL:https://mqtt.waveshare.cloud/http-api-doc.html
    def http_servers_connect(self):
        return self.send_at_command('AT+HTTPPARA="URL",https://www.waveshare.cloud/api/sample-test/')[1].decode(
            'utf-8').strip() == "OK"

    def http_actions(self, enable):
        # enable = 0 is GET method
        # enable = 1 is POST method
        command = f'AT+HTTPACTION={int(enable)}'
        if self.send_at_command(command)[1].decode('utf-8').strip() == "OK":
            self.logresponse_data()

    def http_data(self, data):
        command = f'AT+HTTPDATA={str(len(data))},1000'
        if self.send_at_command(command)[1].decode('utf-8').strip() == "DOWNLOAD":
            if self.send_at_command(data)[1].decode('utf-8').strip() == "OK":
                self.http_actions(1)

    def logresponse_data(self):
        timeout = 3  # Timeout duration in seconds

        start_time = time.time()
        while True:
            response_line = self.ser.readline().decode('utf-8').strip()
            if response_line.startswith("+HTTPACTION"):
                break
            if time.time() - start_time > timeout:
                break
        command = 'AT+HTTPREAD=0,500'
        self.ser.write(command.encode('utf-8') + b'\r\n')
        response = self.ser.readlines()
        if response[1].decode('utf-8').strip() == "OK":
            print(f"The HTTP Response is:{response[4].decode('utf-8').strip()}")

    def destroy(self):
        self.destroy_http()
        self.ser.close()


if __name__ == '__main__':
    try:
        at_instance = AT()
        if at_instance.open_serial_connection():
            print('Opened serial connection')
            at_instance.init_http()
        if at_instance.http_servers_connect():
            print('Network connection established')
            print('Enter data? (yes/no)')
            user_input = input()
            if user_input.lower() == 'y' or user_input.lower() == 'yes':
                print("Enter data:         ---------(Press Enter to finish input!)")
                data = input()
                at_instance.http_data(data)
                while True:
                    print("Continue entering data? (yes/no)")
                    user_input = input()
                    if user_input.lower() == 'y' or user_input.lower() == 'yes':
                        print("Enter data:         ---------(Press Enter to finish input!)")
                        data = input()
                        at_instance.http_data(data)
                    else:
                        at_instance.http_actions(0)
                        at_instance.destroy()
                        break
            elif user_input.lower() == 'no':
                at_instance.http_actions(0)
                at_instance.destroy()
            else:
                at_instance.destroy()
                print("Invalid input!")
        else:
            print('Network connection disabled')
    except KeyboardInterrupt:
        print("KeyboardInterrupt received. Cleaning up before exiting.")
        AT().destroy()
    except Exception as e:
        print(f"An error occurred: {e}")
    finally:
        print("Program terminated.")
